@@ -35,19 +35,41 @@ from hy.models.float import HyFloat
 from hy.models.list import HyList
 from hy.models.dict import HyDict
 
-import hy.importer
-from hy.core import process
-
+from hy.macros import require, process
 from hy.util import str_type
+import hy.importer
 
-import codecs
 import traceback
+import importlib
+import codecs
 import ast
 import sys
 
 from collections import defaultdict
 
 
+_compile_time_ns = {}
+
+
+def compile_time_ns(module_name):
+    ns = _compile_time_ns.get(module_name)
+    if ns is None:
+        ns = {'hy': hy, '__name__': module_name}
+        _compile_time_ns[module_name] = ns
+    return ns
+
+
+_stdlib = {}
+
+
+def load_stdlib():
+    import hy.core
+    for module in hy.core.STDLIB:
+        mod = importlib.import_module(module)
+        for e in mod.EXPORTS:
+            _stdlib[e] = module
+
+
 class HyCompileError(HyError):
     def __init__(self, exception, traceback=None):
         self.exception = exception
@@ -97,7 +119,7 @@ def builds(_type):
 
     unpythonic_chars = ["-"]
     really_ok = ["-"]
-    if True in (x in str_type(_type) for x in unpythonic_chars):
+    if any(x in unpythonic_chars for x in str_type(_type)):
         if _type not in really_ok:
             raise TypeError("`build' needs to be *post* translated strings, "
                             "Mr. / Mrs. Hypser. -- `%s' sucks." % (_type))
@@ -321,10 +343,14 @@ def checkargs(exact=None, min=None, max=None):
 
 class HyASTCompiler(object):
 
-    def __init__(self):
+    def __init__(self, module_name):
         self.anon_fn_count = 0
         self.anon_var_count = 0
         self.imports = defaultdict(set)
+        self.module_name = module_name
+        if not module_name.startswith("hy.core"):
+            # everything in core needs to be explicit.
+            load_stdlib()
 
     def get_anon_var(self):
         self.anon_var_count += 1
@@ -366,7 +392,6 @@ class HyASTCompiler(object):
 
     def compile_atom(self, atom_type, atom):
         if atom_type in _compile_table:
-            atom = process(atom)
             ret = _compile_table[atom_type](self, atom)
             if not isinstance(ret, Result):
                 ret = Result() + ret
@@ -374,6 +399,7 @@ class HyASTCompiler(object):
 
     def compile(self, tree):
         try:
+            tree = process(tree, self.module_name)
             _type = type(tree)
             ret = self.compile_atom(_type, tree)
             if ret:
@@ -419,22 +445,24 @@ class HyASTCompiler(object):
 
             if isinstance(expr, HyLambdaListKeyword):
                 if expr not in expr._valid_types:
-                    raise HyCompileError("{0} is not a valid "
-                                         "lambda-keyword.".format(repr(expr)))
+                    raise HyTypeError(expr, "{0} is not a valid "
+                                      "lambda-keyword.".format(repr(expr)))
                 if expr == "&rest" and lambda_keyword is None:
                     lambda_keyword = expr
                 elif expr == "&optional":
                     if len(defaults) > 0:
-                        raise HyCompileError("There can only be &optional "
-                                             "arguments or one &key argument")
+                        raise HyTypeError(expr,
+                                          "There can only be &optional "
+                                          "arguments or one &key argument")
                     lambda_keyword = expr
                 elif expr == "&key":
                     lambda_keyword = expr
                 elif expr == "&kwargs":
                     lambda_keyword = expr
                 else:
-                    raise HyCompileError("{0} is in an invalid "
-                                         "position.".format(repr(expr)))
+                    raise HyTypeError(expr,
+                                      "{0} is in an invalid "
+                                      "position.".format(repr(expr)))
                 # we don't actually care about this token, so we set
                 # our state and continue to the next token...
                 continue
@@ -443,28 +471,33 @@ class HyASTCompiler(object):
                 args.append(expr)
             elif lambda_keyword == "&rest":
                 if varargs:
-                    raise HyCompileError("There can only be one "
-                                         "&rest argument")
+                    raise HyTypeError(expr,
+                                      "There can only be one "
+                                      "&rest argument")
                 varargs = str(expr)
             elif lambda_keyword == "&key":
                 if type(expr) != HyDict:
-                    raise TypeError("There can only be one &key "
-                                    "argument")
+                    raise HyTypeError(expr,
+                                      "There can only be one &key "
+                                      "argument")
                 else:
                     if len(defaults) > 0:
-                        raise HyCompileError("There can only be &optional "
-                                             "arguments or one &key argument")
+                        raise HyTypeError(expr,
+                                          "There can only be &optional "
+                                          "arguments or one &key argument")
                     # As you can see, Python has a funny way of
                     # defining keyword arguments.
-                    for k, v in expr.items():
+                    it = iter(expr)
+                    for k, v in zip(it, it):
                         args.append(k)
                         ret += self.compile(v)
                         defaults.append(ret.force_expr)
             elif lambda_keyword == "&optional":
                 if isinstance(expr, HyList):
                     if not len(expr) == 2:
-                        raise TypeError("optional args should be bare names "
-                                        "or 2-item lists")
+                        raise HyTypeError(expr,
+                                          "optional args should be bare names "
+                                          "or 2-item lists")
                     k, v = expr
                 else:
                     k = expr
@@ -474,8 +507,9 @@ class HyASTCompiler(object):
                 defaults.append(ret.force_expr)
             elif lambda_keyword == "&kwargs":
                 if kwargs:
-                    raise HyCompileError("There can only be one "
-                                         "&kwargs argument")
+                    raise HyTypeError(expr,
+                                      "There can only be one "
+                                      "&kwargs argument")
                 kwargs = str(expr)
 
         return ret, args, defaults, varargs, kwargs
@@ -543,7 +577,7 @@ class HyASTCompiler(object):
         name = form.__class__.__name__
         imports = set([name])
 
-        if isinstance(form, HyList):
+        if isinstance(form, (HyList, HyDict)):
             if not form:
                 contents = HyList()
             else:
@@ -597,9 +631,10 @@ class HyASTCompiler(object):
     def compile_eval(self, expr):
         expr.pop(0)
 
-        ret = self.compile(HyExpression([
-            HySymbol("hy_eval")] + expr + [
-                HyExpression([HySymbol("locals")])]).replace(expr))
+        ret = self.compile(HyExpression(
+            [HySymbol("hy_eval")] + expr + [HyExpression([HySymbol("locals")])]
+            + [HyString(self.module_name)]).replace(expr)
+        )
 
         ret.add_imports("hy.importer", ["hy_eval"])
 
@@ -760,7 +795,8 @@ class HyASTCompiler(object):
     @builds("except")
     @builds("catch")
     def magic_internal_form(self, expr):
-        raise TypeError("Error: `%s' can't be used like that." % (expr[0]))
+        raise HyTypeError(expr,
+                          "Error: `%s' can't be used like that." % (expr[0]))
 
     def _compile_catch_expression(self, expr, var):
         catch = expr.pop(0)  # catch
@@ -792,6 +828,11 @@ class HyASTCompiler(object):
         # let's pop variable and use it as name
         if len(exceptions) == 2:
             name = exceptions.pop(0)
+            if not isinstance(name, HySymbol):
+                raise HyTypeError(
+                    exceptions,
+                    "Exception storage target name must be a symbol.")
+
             if sys.version_info[0] >= 3:
                 # Python3 features a change where the Exception handler
                 # moved the name from a Name() to a pure Python String type.
@@ -930,6 +971,20 @@ class HyASTCompiler(object):
 
         return ret
 
+    @builds("break")
+    def compile_break_expression(self, expr):
+        ret = ast.Break(lineno=expr.start_line,
+                        col_offset=expr.start_column)
+
+        return ret
+
+    @builds("continue")
+    def compile_continue_expression(self, expr):
+        ret = ast.Continue(lineno=expr.start_line,
+                           col_offset=expr.start_column)
+
+        return ret
+
     @builds("assert")
     @checkargs(1)
     def compile_assert_expression(self, expr):
@@ -1225,6 +1280,7 @@ class HyASTCompiler(object):
         expression.pop(0)
         for entry in expression:
             __import__(entry)  # Import it fo' them macros.
+            require(entry, self.module_name)
         return Result()
 
     @builds("and")
@@ -1369,14 +1425,29 @@ class HyASTCompiler(object):
 
         return ret
 
+    @checkargs(1)
+    def _compile_keyword_call(self, expression):
+        expression.append(expression.pop(0))
+        expression.insert(0, HySymbol("get"))
+        return self.compile(expression)
+
     @builds(HyExpression)
     def compile_expression(self, expression):
+        if expression == []:
+            return self.compile_list(expression)
         fn = expression[0]
         func = None
+        if isinstance(fn, HyKeyword):
+            return self._compile_keyword_call(expression)
+
         if isinstance(fn, HyString):
             ret = self.compile_atom(fn, expression)
             if ret:
                 return ret
+
+            if fn in _stdlib:
+                self.imports[_stdlib[fn]].add(fn)
+
             if fn.startswith("."):
                 # (.split "test test") -> "test test".split()
 
@@ -1623,7 +1694,11 @@ class HyASTCompiler(object):
         ]).replace(expression)
 
         # Compile-time hack: we want to get our new macro now
-        hy.importer.hy_eval(new_expression, {'hy': hy})
+        # We must provide __name__ in the namespace to make the Python
+        # compiler set the __module__ attribute of the macro function.
+        hy.importer.hy_eval(new_expression,
+                            compile_time_ns(self.module_name),
+                            self.module_name)
 
         # We really want to have a `hy` import to get hy.macro in
         ret = self.compile(new_expression)
@@ -1631,6 +1706,23 @@ class HyASTCompiler(object):
 
         return ret
 
+    @builds("eval_and_compile")
+    def compile_eval_and_compile(self, expression):
+        expression[0] = HySymbol("progn")
+        hy.importer.hy_eval(expression,
+                            compile_time_ns(self.module_name),
+                            self.module_name)
+        expression.pop(0)
+        return self._compile_branch(expression)
+
+    @builds("eval_when_compile")
+    def compile_eval_when_compile(self, expression):
+        expression[0] = HySymbol("progn")
+        hy.importer.hy_eval(expression,
+                            compile_time_ns(self.module_name),
+                            self.module_name)
+        return Result()
+
     @builds(HyInteger)
     def compile_integer(self, number):
         return ast.Num(n=int(number),
@@ -1685,7 +1777,7 @@ class HyASTCompiler(object):
 
     @builds(HyDict)
     def compile_dict(self, m):
-        keyvalues, ret = self._compile_collect(sum(m.items(), ()))
+        keyvalues, ret = self._compile_collect(m)
 
         ret += ast.Dict(lineno=m.start_line,
                         col_offset=m.start_column,
@@ -1694,7 +1786,7 @@ class HyASTCompiler(object):
         return ret
 
 
-def hy_compile(tree, root=ast.Module, get_expr=False):
+def hy_compile(tree, module_name, root=ast.Module, get_expr=False):
     """
     Compile a HyObject tree into a Python AST Module.
 
@@ -1711,7 +1803,7 @@ def hy_compile(tree, root=ast.Module, get_expr=False):
     expr = None
 
     if tree:
-        compiler = HyASTCompiler()
+        compiler = HyASTCompiler(module_name)
         result = compiler.compile(tree)
         expr = result.force_expr
 
