@@ -20,49 +20,92 @@
 
 from hy.models.expression import HyExpression
 from hy.models.string import HyString
-from hy.models.dict import HyDict
+from hy.models.symbol import HySymbol
 from hy.models.list import HyList
+from hy.models.integer import HyInteger
+from hy.models.float import HyFloat
+from hy.models.complex import HyComplex
+from hy.models.dict import HyDict
+from hy.util import str_type
 
-_hy_macros = {}
+from collections import defaultdict
+
+
+MACROS = [
+    "hy.core.bootstrap",
+]
+
+_hy_macros = defaultdict(dict)
 
 
 def macro(name):
     def _(fn):
-        _hy_macros[name] = fn
+        module_name = fn.__module__
+        if module_name.startswith("hy.core"):
+            module_name = None
+        _hy_macros[module_name][name] = fn
         return fn
     return _
 
 
-def process(tree):
+def require(source_module_name, target_module_name):
+    macros = _hy_macros[source_module_name]
+    refs = _hy_macros[target_module_name]
+    for name, macro in macros.items():
+        refs[name] = macro
+
+
+def _wrap_value(x):
+    wrapper = _wrappers.get(type(x))
+    if wrapper is None:
+        return x
+    else:
+        return wrapper(x)
+
+_wrappers = {
+    int: HyInteger,
+    bool: lambda x: HySymbol("True") if x else HySymbol("False"),
+    float: HyFloat,
+    complex: HyComplex,
+    str_type: HyString,
+    dict: lambda d: HyDict(_wrap_value(x) for x in sum(d.items(), ())),
+    list: lambda l: HyList(_wrap_value(x) for x in l)
+}
+
+
+def process(tree, module_name):
+    load_macros()
+    old = None
+    while old != tree:
+        old = tree
+        tree = macroexpand(tree, module_name)
+    return tree
+
+
+def load_macros():
+    for module in MACROS:
+        __import__(module)
+
+
+def macroexpand(tree, module_name):
     if isinstance(tree, HyExpression):
+        if tree == []:
+            return tree
+
         fn = tree[0]
         if fn in ("quote", "quasiquote"):
             return tree
-        ntree = HyExpression([fn] + [process(x) for x in tree[1:]])
+        ntree = HyExpression(tree[:])
         ntree.replace(tree)
 
         if isinstance(fn, HyString):
-            if fn in _hy_macros:
-                m = _hy_macros[fn]
-                obj = m(*ntree[1:])
+            m = _hy_macros[module_name].get(fn)
+            if m is None:
+                m = _hy_macros[None].get(fn)
+            if m is not None:
+                obj = _wrap_value(m(*ntree[1:]))
                 obj.replace(tree)
                 return obj
 
-        ntree.replace(tree)
         return ntree
-
-    if isinstance(tree, HyDict):
-        obj = HyDict(dict((process(x), process(tree[x])) for x in tree))
-        obj.replace(tree)
-        return obj
-
-    if isinstance(tree, HyList):
-        obj = HyList([process(x) for x in tree])  # NOQA
-        # flake8 thinks we're redefining from 52.
-        obj.replace(tree)
-        return obj
-
-    if isinstance(tree, list):
-        return [process(x) for x in tree]
-
     return tree
